import { RouteDecorators } from "../lib/decorator"
import * as DecoratorSpec from "../spec/decorator"
import { Errors } from "../spec/errors"

export const AccountDecorators = new class extends DecoratorSpec.AccountDecorators {
    constructor() {
        super()
        this.implement("WithAccount", (context, option, pass) => {
            if (context.req.headers["x-auth-token"]) {
                let token = context.req.headers["x-auth-token"] as string
                context.services.AuthenticationService.getTokenInfo({ token }, (err, info) => {
                    if (err) {
                        context.error(new Errors.Forbidden)
                        return
                    }
                    context.services.AccountService.get(info.account.id, (err, account) => {
                        if (err) {
                            context.error(new Errors.Forbidden("Can't find existing account"))
                            return
                        }
                        // Still set account in this case.
                        // But we respect token more then session.account.
                        context.req.session.account = account
                        context.info.account = account
                        pass()
                    })
                })
            } else {
                if (!context.req.session || !context.req.session.account) {
                    context.error(new Errors.Forbidden)
                    return
                }
                let id = context.req.session.account.id
                context.services.AccountService.get(id, (err, account) => {
                    if (err) {
                        context.error(new Errors.Forbidden("Can't find existing account"))
                        return
                    }
                    context.req.session.account = account
                    context.info.account = account
                    pass()
                })
            }
        })
    }
}
