"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var colors = require("colors/safe");
var useColors = colors;
var DependencyManager = /** @class */ (function () {
    function DependencyManager() {
        this.context = {};
        this.befores = {};
        this.onAddDymanicDependency = function (name) { };
    }
    DependencyManager.prototype.add = function (name, deps, before) {
        if (this.context[name])
            throw new Error("Duplicate dependency entry: " + name);
        if (deps && deps.length == 0)
            deps = null;
        this.context[name] = deps;
        if (before) {
            this.befores[name] = before;
        }
    };
    DependencyManager.prototype.addReverseDependency = function (name, dep) {
        if (!this.context[name])
            this.context[name] = [];
        this.context[name].push(dep);
        console.log(useColors.green("Add reverse dependency"), name, useColors.green("depend on"), dep);
    };
    DependencyManager.prototype.resolveDependency = function (name, chain) {
        if (chain === void 0) { chain = []; }
        // inject all before into dependency
        for (var name_1 in this.befores) {
            var matchers = this.befores[name_1];
            for (var serviceName in this.context) {
                for (var _i = 0, matchers_1 = matchers; _i < matchers_1.length; _i++) {
                    var matcher = matchers_1[_i];
                    if (typeof matcher == "string") {
                        if (matcher === serviceName) {
                            this.addReverseDependency(serviceName, name_1);
                            break;
                        }
                    }
                    else if (matcher instanceof RegExp && matcher.test(serviceName)) {
                        this.addReverseDependency(serviceName, name_1);
                        break;
                    }
                }
            }
        }
        if (typeof this.context[name] === "undefined") {
            this.onAddDymanicDependency(name);
            if (typeof this.context[name] === "undefined") {
                throw new Error(name + " required by " + chain[chain.length - 1] + " couldn't be fullfilled.");
            }
        }
        if (this.context[name] === null)
            return [];
        chain = chain.slice();
        if (chain.indexOf(name) >= 0) {
            chain.push(name);
            throw new Error("Recursive dependency detected: " + chain.join("\n=>"));
        }
        chain.push(name);
        var result = [];
        var all = [];
        for (var _a = 0, _b = this.context[name] || []; _a < _b.length; _a++) {
            var matcher = _b[_a];
            if (typeof matcher === "string") {
                if (all.indexOf(matcher) < 0)
                    all.push(matcher);
            }
            else if (matcher instanceof RegExp) {
                for (var name_2 in this.context) {
                    if (matcher.test(name_2) && all.indexOf(name_2) < 0) {
                        all.push(name_2);
                    }
                }
            }
        }
        for (var _c = 0, all_1 = all; _c < all_1.length; _c++) {
            var dep = all_1[_c];
            if (dep === name)
                continue;
            var deps = this.resolveDependency(dep, chain);
            for (var _d = 0, deps_1 = deps; _d < deps_1.length; _d++) {
                var subDep = deps_1[_d];
                if (result.indexOf(subDep) < 0)
                    result.push(subDep);
            }
            result.push(dep);
        }
        return result;
    };
    DependencyManager.prototype.resolve = function () {
        var result = [];
        for (var name_3 in this.context) {
            var deps = this.resolveDependency(name_3);
            for (var _i = 0, deps_2 = deps; _i < deps_2.length; _i++) {
                var dep = deps_2[_i];
                if (result.indexOf(dep) < 0)
                    result.push(dep);
            }
            if (result.indexOf(name_3) < 0)
                result.push(name_3);
        }
        return result;
    };
    return DependencyManager;
}());
exports.DependencyManager = DependencyManager;
//# sourceMappingURL=dependency.js.map