import * as Reg from "source-map-support/register"
import * as colors from "colors/safe"
let useColors = colors

export type DependencyMatcher = string | RegExp
export class DependencyManager {
    private context: { [index: string]: DependencyMatcher[] } = {}
    add(name: string, deps: DependencyMatcher[], before?: DependencyMatcher[]): void {
        if (this.context[name]) throw new Error(`Duplicate dependency entry: ${name}`)
        if (deps && deps.length == 0) deps = null
        this.context[name] = deps
        if (before) {
            this.befores[name] = before
        }
    }
    befores: {
        [name: string]: DependencyMatcher[]
    } = {}
    private addReverseDependency(name: string, dep: string) {
        if (!this.context[name]) this.context[name] = []
        this.context[name].push(dep)
        console.log(useColors.green("Add reverse dependency"), name, useColors.green("depend on"), dep)
    }
    onAddDymanicDependency: (name: string) => void = (name: string) => { }
    resolveDependency(name: string, chain: string[] = []): string[] {
        // inject all before into dependency
        for (let name in this.befores) {
            let matchers = this.befores[name]
            for (let serviceName in this.context) {
                for (let matcher of matchers) {
                    if (typeof matcher == "string") {
                        if (matcher === serviceName) {
                            this.addReverseDependency(serviceName, name)
                            break
                        }
                    } else if (matcher instanceof RegExp && matcher.test(serviceName)) {
                        this.addReverseDependency(serviceName, name)
                        break
                    }
                }
            }
        }




        if (typeof this.context[name] === "undefined") {
            this.onAddDymanicDependency(name)
            if (typeof this.context[name] === "undefined") {
                throw new Error(`${name} required by ${chain[chain.length - 1]} couldn't be fullfilled.`)
            }
        }
        if (this.context[name] === null) return []
        chain = chain.slice()
        if (chain.indexOf(name) >= 0) {
            chain.push(name)
            throw new Error(`Recursive dependency detected: ${chain.join("\n=>")}`)
        }
        chain.push(name)
        let result: string[] = []
        let all: string[] = []
        for (let matcher of this.context[name] || []) {
            if (typeof matcher === "string") {
                if (all.indexOf(matcher) < 0) all.push(matcher)
            } else if (matcher instanceof RegExp) {
                for (let name in this.context) {
                    if (matcher.test(name) && all.indexOf(name) < 0) {
                        all.push(name)
                    }
                }
            }
        }
        for (let dep of all) {
            if (dep === name) continue
            let deps = this.resolveDependency(dep, chain)
            for (let subDep of deps) {
                if (result.indexOf(subDep) < 0) result.push(subDep)
            }
            result.push(dep)
        }
        return result
    }
    resolve(): string[] {
        let result: string[] = []
        for (let name in this.context) {
            let deps = this.resolveDependency(name)
            for (let dep of deps) {
                if (result.indexOf(dep) < 0) result.push(dep)
            }
            if (result.indexOf(name) < 0) result.push(name)
        }
        return result
    }
}
