/// <reference types="express" />
import { Service } from "./service";
import * as express from "express";
import { RouteDecorator } from "./decorator";
export interface Request extends express.Request {
}
export interface Response extends express.Response {
}
export declare abstract class RouteService<T extends APIServiceSpec = any> extends Service {
    spec: T;
    routes: RouteConstructor[];
    prefix: string;
    constructor(spec?: T);
    addRoute(RouteConstructor: any): void;
    decorateAll(...decs: RouteDecorator[]): this;
    clearDecorateAll(): void;
    private commonDecorateStacks;
    private decorateStacks;
    decorate(...decs: RouteDecorator[]): this;
    all<Self extends this = this>(option: {
        [K in keyof T]: {
            decorators?: RouteDecorator[];
            handler: (this: T[K] & APIRoute<T[K]["return"], T[K]["error"], Self>, context?: T[K] & APIRoute<T[K]["return"], T[K]["error"], Self>) => void;
        };
    }): void;
    api<K extends keyof T = string>(name: K, ...callbacks: ((this: T[K] & APIRoute<T[K]["return"], T[K]["error"], this>, context?: T[K] & APIRoute<T[K]["return"], T[K]["error"], this>) => void)[]): APIRoute<T[K]["return"], T[K]["error"], any> & T[K];
    getAPI<K extends keyof T>(name: K): APIRoute<T[K]["return"]>;
    getRoute(name: keyof T): Route<any>;
    defaultMethod: METHOD_ENUM;
    installTo(server: express.Application): void;
    addRoutes(...obj: any[]): void;
    isRouteConstructor(obj: any): boolean;
}
export declare abstract class RouteServiceTemplate<T extends APIServiceSpec = any> extends RouteService<T> {
    name: string;
    constructor(spec: {
        API: T;
        prefix?: string;
        name: string;
    });
}
export interface RouteConstructor {
    new (service?: RouteService): Route<any>;
}
export declare abstract class Route<TMIME extends MIME, TService extends Service = any> {
    service: TService;
    static decorate(fn: RouteDecorator): typeof Route;
    static decorators: RouteDecorator[];
    constructor(service: TService);
    decorate(...decs: RouteDecorator<this>[]): void;
    name: string;
    mime: TMIME;
    abstract path: string;
    method: METHOD_ENUM;
    params: {};
    body: {};
    info: {};
    services: TService["services"];
    query: {};
    next: Function;
    req: Request;
    res: Response;
    handleRawRequest(req: any, res: any, next: any): void;
    end(content: any): void;
    abstract handle(): any;
    notFound(content?: string, mime?: TMIME): void;
}
export declare abstract class DynamicResourceRoute<T extends MIME = string> extends Route<T> {
}
export declare abstract class MiddlewareRoute extends Route<never> {
    path: any;
    method: Method.ALL;
    mock(url: string, data: {
        [name: string]: any;
    }, callback: (err?: any) => void): void;
}
export declare abstract class PageRoute extends Route<"text/html"> {
    method: Method.GET;
    readonly mime: string;
    html(content: string): void;
}
export declare abstract class APIRoute<TResponse, TError extends Error = any, TService extends Service = any> extends Route<"application/json", TService> {
    done(err: TError, result: TResponse): void;
    success(data: TResponse): void;
    error(err: TError): void;
    _error: (err: TError) => void;
    _success: (data: any) => void;
    json(data: any): void;
    mock(url: string, data: {
        [name: string]: any;
    }, callback: (err?: TError, data?: TResponse) => void): void;
}
export declare type MIME = "text/html" | "text/javascript" | "text/json" | "application/json" | "application/octet-stream" | string;
export declare type METHOD_ENUM = "GET" | "POST" | "DELETE" | "PUT" | "ALL";
export declare type Method<T extends METHOD_ENUM> = string & T;
export declare namespace Method {
    const GET: Method<"GET">;
    const POST: Method<"POST">;
    const DELETE: Method<"DELETE">;
    const PUT: Method<"PUT">;
    const ALL: Method<"ALL">;
    type POST = Method<"POST">;
    type GET = Method<"GET">;
    type DELETE = Method<"DELETE">;
    type PUT = Method<"PUT">;
    type ALL = Method<"ALL">;
}
export declare type APITemplate = APISpec;
export interface APISpec {
    path: string;
    method?: METHOD_ENUM;
    return: any;
    error?: Error;
    service?: RouteService;
}
export interface APIServiceSpec {
    [name: string]: APISpec;
}
export declare abstract class APIBase implements APITemplate {
    abstract path: string;
    return: any;
    error: Error;
    method?: METHOD_ENUM;
}
