"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("./service");
var urlModule = require("url");
var colors = require("colors");
var PathToRegExp = require("path-to-regexp");
var useColors = colors;
var RouteService = /** @class */ (function (_super) {
    __extends(RouteService, _super);
    function RouteService(spec, defaultSettings, settings) {
        var _this = _super.call(this, defaultSettings || {}, settings || {}) || this;
        _this.spec = spec;
        _this.routes = [];
        _this.prefix = "/";
        _this.dependencies = ["ExpressService"];
        _this.commonDecorateStacks = [];
        _this.decorateStacks = [];
        _this.defaultMethod = "POST";
        _this.isInstalled = false;
        return _this;
    }
    RouteService.prototype.afterInitialize = function () {
        if (!this.isInstalled && this.services.ExpressService) {
            this.installTo(this.services.ExpressService.server);
        }
    };
    RouteService.prototype.decorateAPI = function (name, dec) {
        for (var _i = 0, _a = this.routes; _i < _a.length; _i++) {
            var route = _a[_i];
            if (route.apiName == name) {
                route.decorate(dec);
            }
        }
    };
    RouteService.prototype.addRoute = function (RouteConstructor) {
        this.routes.push(RouteConstructor);
    };
    RouteService.prototype.decorateAll = function () {
        var decs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            decs[_i] = arguments[_i];
        }
        (_a = this.commonDecorateStacks).push.apply(_a, decs);
        return this;
        var _a;
    };
    RouteService.prototype.clearDecorateAll = function () {
        this.commonDecorateStacks.length = 0;
    };
    RouteService.prototype.decorate = function () {
        var decs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            decs[_i] = arguments[_i];
        }
        (_a = this.decorateStacks).push.apply(_a, decs);
        return this;
        var _a;
    };
    RouteService.prototype.all = function (option) {
        for (var key in option) {
            if (option[key].decorators) {
                this.decorate.apply(this, option[key].decorators);
            }
            this.api(key, option[key].handler);
        }
    };
    RouteService.prototype.api = function (name) {
        var callbacks = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            callbacks[_i - 1] = arguments[_i];
        }
        if (!this.spec) {
            throw new Error("No spec provided");
        }
        if (!this.spec[name]) {
            throw new Error("Unknown spec" + name);
        }
        var service = this;
        var callback = callbacks.pop();
        var Cons = (_a = /** @class */ (function (_super) {
                __extends(class_1, _super);
                function class_1(arg) {
                    var _this = _super.call(this, arg) || this;
                    _this.name = name;
                    _this.path = service.spec[name].path;
                    _this.method = service.spec[name].method || service.defaultMethod;
                    var decs = Cons.decorators;
                    _this.decorate.apply(_this, decs);
                    return _this;
                }
                class_1.prototype.handle = function () {
                    callback.call(this, this);
                };
                return class_1;
            }(APIRoute)),
            _a.apiName = name,
            _a);
        Cons.decorators = [];
        (_b = Cons.decorators).push.apply(_b, this.commonDecorateStacks);
        (_c = Cons.decorators).push.apply(_c, this.decorateStacks.slice(0));
        this.decorateStacks.length = 0;
        this.addRoute(Cons);
        return Cons;
        var _a;
        var _b, _c;
    };
    RouteService.prototype.getAPI = function (name) {
        return this.getRoute(name);
    };
    RouteService.prototype.getRoute = function (name) {
        for (var _i = 0, _a = this.routes; _i < _a.length; _i++) {
            var Cons = _a[_i];
            var route = new Cons(this);
            if (route.name == name)
                return route;
        }
        return null;
    };
    RouteService.prototype.installTo = function (server) {
        var _this = this;
        if (!server)
            return;
        this.isInstalled = true;
        var _loop_1 = function (cons) {
            var route = new cons(this_1);
            if (!route.path)
                return "continue";
            if (route instanceof MiddlewareRoute) {
                var handler = function (req, res, next) {
                    var route = new cons(_this);
                    route.services = _this.services;
                    route.handleRawRequest(req, res, next);
                };
                if (route.path) {
                    server.use(route.path, handler);
                }
                else {
                    server.use(handler);
                }
                console.log("Install middleware " + useColors.yellow(cons.name));
                return "continue";
            }
            var path = urlModule.resolve(this_1.prefix, route.path);
            var method = route.method.toLowerCase();
            if (route.name) {
                console.log(useColors.yellow(route.method) + " => " + useColors.yellow(path) + " ~ Install " + useColors.yellow(route.name));
            }
            else {
                console.log("Install " + useColors.yellow(route.method) + " => " + useColors.yellow(path) + " ");
            }
            server[method](path, function (req, res, next) {
                var route = new cons(_this);
                route.services = _this.services;
                route.handleRawRequest(req, res, next);
            });
        };
        var this_1 = this;
        for (var _i = 0, _a = this.routes; _i < _a.length; _i++) {
            var cons = _a[_i];
            _loop_1(cons);
        }
    };
    RouteService.prototype.addRoutes = function () {
        var obj = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            obj[_i] = arguments[_i];
        }
        for (var _a = 0, obj_1 = obj; _a < obj_1.length; _a++) {
            var item = obj_1[_a];
            if (this.isRouteConstructor(item)) {
                this.addRoute(item);
            }
            else {
                for (var name_1 in item) {
                    var target = item[name_1];
                    if (this.isRouteConstructor(target)) {
                        this.addRoute(target);
                    }
                }
            }
        }
    };
    RouteService.prototype.isRouteConstructor = function (obj) {
        return obj && obj.prototype && obj.prototype["thisObjectIsTruelyARoute"] == "thisObjectIsTruelyARoute";
    };
    return RouteService;
}(service_1.Service));
exports.RouteService = RouteService;
var RouteServiceTemplate = /** @class */ (function (_super) {
    __extends(RouteServiceTemplate, _super);
    function RouteServiceTemplate(spec) {
        var _this = _super.call(this, spec.API) || this;
        _this.name = spec.name;
        _this.prefix = spec.prefix || "";
        return _this;
    }
    return RouteServiceTemplate;
}(RouteService));
exports.RouteServiceTemplate = RouteServiceTemplate;
var Route = /** @class */ (function () {
    function Route(service) {
        var _this = this;
        this.service = service;
        this.method = "GET";
        this.params = {};
        this.body = {};
        this.info = {};
        this.query = {};
        if (!this["handle"]) {
            this["handle"] = function () {
                _this.res.json({
                    state: false,
                    error: new Error("Not implemented"),
                    data: "Not implemented"
                });
            };
        }
    }
    Route.decorate = function (fn) {
        this.decorators.push(fn);
        return this;
    };
    Route.prototype.decorate = function () {
        var decs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            decs[_i] = arguments[_i];
        }
        var dec;
        decs = decs.slice();
        while (dec = decs.pop()) {
            var des = { value: this.handle };
            dec(this, "handle", des);
            if (des.value != this.handle) {
                this.handle = des.value;
            }
        }
    };
    Route.prototype.handleRawRequest = function (req, res, next) {
        if (!req._routeInfo) {
            req._routeInfo = {};
        }
        this.req = req;
        this.res = res;
        this.info = req._routeInfo;
        this.next = next;
        this.body = req.body || {};
        this.params = req.params || {};
        this.query = req.query || {};
        this.handle();
    };
    Route.prototype.end = function (content) {
        this.res.contentType(this.mime);
        this.res.end(content);
    };
    Route.prototype.notFound = function (content, mime) {
        this.mime = mime || "text/plaintext";
        this.res.statusCode = 404;
        this.res.end(content || "404");
    };
    Route.decorators = [];
    return Route;
}());
exports.Route = Route;
Route.prototype["thisObjectIsTruelyARoute"] = "thisObjectIsTruelyARoute";
var DynamicResourceRoute = /** @class */ (function (_super) {
    __extends(DynamicResourceRoute, _super);
    function DynamicResourceRoute() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return DynamicResourceRoute;
}(Route));
exports.DynamicResourceRoute = DynamicResourceRoute;
var MiddlewareRoute = /** @class */ (function (_super) {
    __extends(MiddlewareRoute, _super);
    function MiddlewareRoute() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.path = null;
        return _this;
    }
    MiddlewareRoute.prototype.mock = function (url, data, callback) {
        for (var name_2 in data) {
            this[name_2] = data[name_2];
        }
        var keys = [];
        if (this.path) {
            var reg = PathToRegExp(this.path, keys, {});
            var match = reg.exec(url);
            if (!match) {
                callback(new Error(url + " doesn't match " + this.path));
                return;
            }
            var params = {};
            for (var i = 0; i < keys.length; i++) {
                var k = keys[i];
                params[k.name] = match[i + 1];
            }
            this.params = params;
            if (!reg.test(url)) {
                callback(new Error("Url not match"));
                return;
            }
        }
        this.next = function () {
            callback();
        };
        this.handle();
    };
    return MiddlewareRoute;
}(Route));
exports.MiddlewareRoute = MiddlewareRoute;
var PageRoute = /** @class */ (function (_super) {
    __extends(PageRoute, _super);
    function PageRoute() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.mime = "text/html";
        return _this;
    }
    PageRoute.prototype.html = function (content) {
        this.end(content);
    };
    return PageRoute;
}(Route));
exports.PageRoute = PageRoute;
var APIRoute = /** @class */ (function (_super) {
    __extends(APIRoute, _super);
    function APIRoute(service) {
        var _this = _super.call(this, service) || this;
        _this.done = _this.done.bind(_this);
        _this.error = _this.error.bind(_this);
        _this.success = _this.success.bind(_this);
        _this.ok = _this.success;
        return _this;
    }
    APIRoute.prototype.done = function (err, result) {
        if (err) {
            this.error(err);
        }
        else {
            this.success(result);
        }
    };
    APIRoute.prototype.ok = function (data) {
        return this.success(data);
    };
    APIRoute.prototype.success = function (data) {
        if (this._success) {
            this._success(data);
            return;
        }
        this.json({
            state: true,
            data: data
        });
    };
    APIRoute.prototype.error = function (err) {
        if (this._error) {
            this._error(err);
            return;
        }
        this.json({
            state: false,
            error: err
        });
    };
    APIRoute.prototype.json = function (data) {
        this.res.json(data);
    };
    APIRoute.prototype.mock = function (url, data, callback) {
        var _this = this;
        for (var name_3 in data) {
            this[name_3] = data[name_3];
        }
        var keys = [];
        var reg = PathToRegExp(this.path, keys, {});
        var match = reg.exec(url);
        if (!match) {
            callback(new Error(url + " doesn't match " + this.path));
            return;
        }
        var params = {};
        for (var i = 0; i < keys.length; i++) {
            var k = keys[i];
            params[k.name] = match[i + 1];
        }
        this.params = params;
        if (!reg.test(url)) {
            callback(new Error("Url not match"));
            return;
        }
        var returnError = null;
        var returnValue = null;
        this._error = function (err) {
            clear();
            callback(err, null);
        };
        this._success = function (data) {
            clear();
            callback(null, data);
        };
        var clear = function () {
            _this._error = null;
            _this._success = null;
        };
        this.handle();
    };
    APIRoute.apiName = "null";
    return APIRoute;
}(Route));
exports.APIRoute = APIRoute;
var Method;
(function (Method) {
    Method.GET = "GET";
    Method.POST = "POST";
    Method.DELETE = "DELETE";
    Method.PUT = "PUT";
    Method.ALL = "ALL";
})(Method = exports.Method || (exports.Method = {}));
var APIBase = /** @class */ (function () {
    function APIBase() {
    }
    return APIBase;
}());
exports.APIBase = APIBase;
//# sourceMappingURL=route.js.map