"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var RouteSpec = require("../spec/route");
var errors_1 = require("../spec/errors");
var account_1 = require("../decorator/account");
var AccountAPIService = /** @class */ (function (_super) {
    __extends(AccountAPIService, _super);
    function AccountAPIService() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccountAPIService.prototype.initialize = function () {
        var _this = this;
        this.api("getTokenStandard", function (context) {
            if (!context.body.password || !context.body.login) {
                context.error(new errors_1.Errors.InvalidParameter);
                return;
            }
            var meta = {
                password: context.body.password
            };
            var login = context.body.login;
            if (login.indexOf("@") >= 0) {
                meta.email = login;
            }
            else {
                meta.name = login;
            }
            context.services.AuthenticationService.authenticate({
                authentication: {
                    type: "common",
                    meta: meta
                }
            }, function (err, auth) {
                if (err) {
                    context.error(new errors_1.Errors.AuthorizationFailed("Invalid auth"));
                    return;
                }
                context.services.AccountService.get(auth.accountId, function (err, account) {
                    if (err) {
                        context.error(new errors_1.Errors.AuthorizationFailed("Missing account"));
                        return;
                    }
                    context.services.AuthenticationService.createToken({
                        account: account
                    }, function (err, token) {
                        if (err) {
                            context.error(err);
                            return;
                        }
                        context.success({
                            token: token,
                            account: account
                        });
                    });
                });
            });
        });
        this.api("webLoginStandard", function (context) {
            if (!context.body.password || !context.body.login) {
                context.error(new errors_1.Errors.InvalidParameter);
                return;
            }
            var meta = {
                password: context.body.password
            };
            var login = context.body.login;
            if (login.indexOf("@") >= 0) {
                meta.email = login;
            }
            else {
                meta.name = login;
            }
            context.services.AuthenticationService.authenticate({
                authentication: {
                    type: "common",
                    meta: meta
                }
            }, function (err, auth) {
                if (err) {
                    context.error(new errors_1.Errors.AuthorizationFailed);
                    return;
                }
                context.services.AccountService.get(auth.accountId, function (err, account) {
                    if (err) {
                        context.error(new errors_1.Errors.AuthorizationFailed("Missing account"));
                        return;
                    }
                    context.req.session["account"] = account;
                    context.success({
                        account: account
                    });
                });
            });
        });
        this.api("registerStandard", function (context) {
            _this.services.AccountService.register({
                type: "common",
                meta: {
                    active: true,
                    displayName: context.body.name,
                    password: context.body.password,
                    email: context.body.email,
                    name: context.body.name,
                }
            }, function (err, account) {
                if (err) {
                    context.error(err);
                    return;
                }
                context.req.session["account"] = account;
                context.success({
                    account: account
                });
            });
        });
        this.decorate(account_1.AccountDecorators.decorate("WithAccount")).api("getCurrentAccount", function (context) {
            context.success(context.info.account);
        });
        this.installTo(this.services.ExpressService.server);
    };
    return AccountAPIService;
}(RouteSpec.AccountAPIServiceSpec));
exports.AccountAPIService = AccountAPIService;
//# sourceMappingURL=api.account.js.map