import * as RouteSpec from "../spec/route"
import { Errors } from "../spec/errors"
import * as Leaf from "leaf-ts"
import * as async from "async"
import { RouteDecorators } from "../lib/decorator"
import { AccountDecorators } from "../decorator/account"
export class AccountAPIService extends RouteSpec.AccountAPIServiceSpec {
    initialize() {
        this.api("getTokenStandard", (context) => {
            if (!context.body.password || !context.body.login) {
                context.error(new Errors.InvalidParameter)
                return
            }
            let meta = {
                password: context.body.password
            } as any
            let login = context.body.login
            if (login.indexOf("@") >= 0) {
                meta.email = login
            } else {
                meta.name = login
            }
            context.services.AuthenticationService.authenticate({
                authentication: {
                    type: "common",
                    meta: meta
                }
            }, (err, auth) => {
                if (err) {
                    context.error(new Errors.AuthorizationFailed("Invalid auth"))
                    return
                }
                context.services.AccountService.get(auth.accountId, (err, account) => {
                    if (err) {
                        context.error(new Errors.AuthorizationFailed("Missing account"))
                        return
                    }
                    context.services.AuthenticationService.createToken({
                        account
                    }, (err, token) => {
                        if (err) {
                            context.error(err)
                            return
                        }
                        context.success({
                            token,
                            account: account
                        })
                    })
                })
            })

        })
        this.api("webLoginStandard", (context) => {
            if (!context.body.password || !context.body.login) {
                context.error(new Errors.InvalidParameter)
                return
            }
            let meta = {
                password: context.body.password
            } as any
            let login = context.body.login
            if (login.indexOf("@") >= 0) {
                meta.email = login
            } else {
                meta.name = login
            }
            context.services.AuthenticationService.authenticate({
                authentication: {
                    type: "common",
                    meta: meta
                }
            }, (err, auth) => {
                if (err) {
                    context.error(new Errors.AuthorizationFailed)
                    return
                }
                context.services.AccountService.get(auth.accountId, (err, account) => {
                    if (err) {
                        context.error(new Errors.AuthorizationFailed("Missing account"))
                        return
                    }
                    context.req.session["account"] = account
                    context.success({
                        account: account
                    })
                })
            })
        })
        this.api("registerStandard", (context) => {
            this.services.AccountService.register({
                type: "common",
                meta: {
                    active: true,
                    displayName: context.body.name,
                    password: context.body.password,
                    email: context.body.email,
                    name: context.body.name,
                }
            }, (err, account) => {
                if (err) {
                    context.error(err)
                    return
                }
                context.req.session["account"] = account
                context.success({
                    account: account
                })
            })
        })
        this.decorate(AccountDecorators.decorate("WithAccount")).api("getCurrentAccount", (context) => {
            context.success(context.info.account)
        })
        this.installTo(this.services.ExpressService.server)
    }
}
