"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var uuid = require("uuid/v4");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.build({
    NotFound: {},
    UnknownError: {},
    DBError: {},
    AlreadyExists: {},
    InvalidParameter: {},
    NameAlreadyExists: {},
    EmailAlreadyExists: {},
});
var AccountService = /** @class */ (function (_super) {
    __extends(AccountService, _super);
    function AccountService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.name = "AccountService";
        _this.collectionName = "account";
        _this.dependencies = ["MongodbService", "AuthenticationService"];
        return _this;
    }
    AccountService.prototype.initialize = function (done) {
        this.accountCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.accountCollection.createIndex({
            id: 1
        }, {
            unique: true
        });
        done();
    };
    AccountService.prototype.create = function (option, callback) {
        if (!option.account.id) {
            option.account.id = uuid();
        }
        this.accountCollection.insert(option.account, function (err, result) {
            if (err) {
                callback(new exports.Errors.AlreadyExists);
                return;
            }
            callback(null, option.account);
        });
    };
    AccountService.prototype.createFromAuthentication = function (option, callback) {
        var auth = option.authentication;
        var def = this.services.AuthenticationService.getDefinition(auth.type);
        if (!def) {
            callback(new exports.Errors.InvalidParameter("Unknown authentication type: " + auth.type));
            return;
        }
        var account = {
            id: option.id || uuid(),
            displayName: option.displayName || def.about(auth.meta, "displayName"),
            name: option.name || def.about(auth.meta, "name"),
            props: def.props(auth.meta) || {}
        };
        if (option.props) {
            account.props = __assign({}, account.props, option.props);
        }
        if (option.role || def.about(auth.meta, "role")) {
            account.role = option.role || def.about(auth.meta, "role");
        }
        this.accountCollection.insert(account, function (err, result) {
            if (err) {
                callback(new exports.Errors.AlreadyExists);
                return;
            }
            callback(null, account);
        });
    };
    AccountService.prototype.get = function (id, callback) {
        this.accountCollection.findOne({
            id: id
        }, function (err, obj) {
            if (err || !obj) {
                callback(new exports.Errors.NotFound);
                return;
            }
            delete obj["_id"];
            callback(null, obj);
        });
    };
    AccountService.prototype.gets = function (ids, callback) {
        this.accountCollection.find({
            id: {
                $in: ids
            }
        }).toArray(function (err, accounts) {
            if (err || !accounts) {
                callback(null, []);
                return;
            }
            callback(null, accounts);
        });
    };
    AccountService.prototype.update = function (option, callback) {
        var updates = {
            $set: option.account
        };
        if (option.mongoUpdates) {
            updates = option.mongoUpdates;
        }
        this.accountCollection.findOneAndUpdate({
            id: option.account.id
        }, updates, function (err, result) {
            if (err) {
                callback(new exports.Errors.DBError("Failt to update", { via: err }));
                return;
            }
            var updated = result.value;
            delete updated["_id"];
            callback(null, updated);
        });
    };
    AccountService.prototype.register = function (option, callback) {
        var _this = this;
        var authentication = {
            type: option.type,
            meta: option.meta
        };
        this.services.AuthenticationService.create({
            authentication: authentication
        }, function (err, authentication) {
            if (err) {
                callback(err);
                return;
            }
            _this.createFromAuthentication({
                authentication: authentication,
                role: option.role || "account"
            }, function (err, account) {
                if (err) {
                    callback(err);
                    return;
                }
                authentication.accountId = account.id;
                _this.services.AuthenticationService.update({
                    authentication: authentication
                }, function (err, result) {
                    if (err) {
                        callback(err);
                        return;
                    }
                    callback(null, account);
                });
            });
        });
    };
    return AccountService;
}(service_1.Service));
exports.AccountService = AccountService;
//# sourceMappingURL=accountService.js.map