import { Service } from "../lib/service";
import { MongodbService } from "./mongodbService";
import { TTLSessionStorage } from "./ttlSessionStorage";
import * as mongodb from "mongodb";
export declare type AccountDefinition = AccountService.AccountDefinition;
export declare type AuthenticationData<TType extends string, TMeta> = AuthenticationService.AuthenticationData<TType, TMeta>;
export interface Callback<T> {
    (error?: Error, data?: T): void;
}
export declare class AuthenticationService extends Service<AuthenticationService.Settings> {
    readonly name: string;
    protected collectionName: string;
    readonly dependencies: string[];
    constructor(settings: AuthenticationService.Settings);
    services: {
        MongodbService: MongodbService;
        TTLSessionStorage: TTLSessionStorage;
    };
    authenticationCollection: mongodb.Collection;
    createToken<TAccount extends AccountDefinition = AccountDefinition>(option: {
        account: TAccount;
        ttl?: number;
    }, callback: Callback<string>): void;
    deleteToken(option: {
        token: string;
    }, callback: Callback<null>): void;
    getTokenInfo<TAccount extends AccountDefinition = AccountDefinition>(option: {
        token: string;
    }, callback: Callback<{
        account: TAccount;
        createAt: Date;
    }>): void;
    initialize(done: any): void;
    definitions: {
        [type: string]: AuthenticationDefinition;
    };
    define(definition: AuthenticationDefinition): void;
    create(option: {
        authentication: AuthenticationData;
    }, callback: Callback<AuthenticationData>): void;
    authenticate<TType extends string, TMeta>(option: {
        authentication: Partial<AuthenticationData<TType, TMeta>>;
    }, callback: Callback<AuthenticationData<TType, TMeta>>): void;
    get<TType extends string, TMeta>(option: {
        authentication: Partial<AuthenticationData<TType, TMeta>>;
    }, callback: Callback<AuthenticationData<TType, TMeta>>): void;
    update<TType extends string, TMeta>(option: {
        authentication: AuthenticationData<TType, TMeta>;
    }, callback: Callback<AuthenticationData<TType, TMeta>>): void;
    findByAccountId(accountId: string, callback: Callback<AuthenticationData[]>): void;
    private sanitizeAndGetDefinition(auth);
    getDefinition(type: string): AuthenticationDefinition;
}
export declare abstract class AuthenticationDefinition<TType extends string = any, TMeta = any> {
    abstract type: TType;
    abstract sanitize(meta: TMeta): TMeta;
    abstract queryCriteria(meta: TMeta): any;
    abstract authenticateCriteria(meta: TMeta): any;
    abstract clearSensitiveInformation(meta: TMeta): TMeta;
    abstract guid(meta: TMeta): string;
    abstract about(meta: TMeta, field: "displayName" | string): any;
    abstract unique(): {
        [key: string]: number;
    }[];
    meta: TMeta;
    abstract props(meta: TMeta): any;
    abouts(meta: TMeta, fields: string[]): {};
}
export declare class CommonAuthenticationDefinition extends AuthenticationDefinition<"common", {
    name: string;
    displayName: string;
    email: string;
    password?: string;
    passwordHash?: string;
    active: boolean;
}> {
    option: {
        salt?: string;
        maxNameLength?: number;
        minNameLength?: number;
        maxEmailLength?: number;
        minEmailLength?: number;
        minPasswordLength?: number;
        maxPasswordLength?: number;
        emailValidator?: RegExp;
        nameValidator?: RegExp;
        passwordChecker?: (string) => boolean;
        requireActivation?: boolean;
    };
    constructor(option?: {
        salt?: string;
        maxNameLength?: number;
        minNameLength?: number;
        maxEmailLength?: number;
        minEmailLength?: number;
        minPasswordLength?: number;
        maxPasswordLength?: number;
        emailValidator?: RegExp;
        nameValidator?: RegExp;
        passwordChecker?: (string) => boolean;
        requireActivation?: boolean;
    });
    readonly type: string;
    sanitize(meta: this["meta"]): this["meta"];
    queryCriteria(meta: this["meta"]): any;
    authenticateCriteria(meta: this["meta"]): any;
    props(meta: this["meta"]): {
        email: string;
        name: string;
    };
    clearSensitiveInformation(meta: this["meta"]): this["meta"];
    guid(meta: this["meta"]): string;
    unique(): ({
        "meta.email": number;
        meta.name?: undefined;
    } | {
        "meta.name": number;
        meta.email?: undefined;
    })[];
    about(meta: this["meta"], field: string): any;
    private hash(raw);
}
