"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var Leaf = require("leaf-ts");
var crypto = require("crypto");
var Errors = Leaf.ErrorDoc.build({
    InvalidParameter: {},
    NotFound: {},
    AlreadyExists: {}
});
var AuthenticationService = /** @class */ (function (_super) {
    __extends(AuthenticationService, _super);
    function AuthenticationService(settings) {
        var _this = _super.call(this, {
            authentication: {
                ttl: 1000 * 60 * 60 * 24 * 30
            }
        }, settings) || this;
        _this.name = "AuthenticationService";
        _this.collectionName = "authentication";
        _this.dependencies = ["MongodbService", "TTLSessionStorage"];
        _this.definitions = {};
        _this.define(new CommonAuthenticationDefinition);
        return _this;
    }
    AuthenticationService.prototype.createToken = function (option, callback) {
        this.services.TTLSessionStorage.createTTLSession({
            data: {
                account: option.account,
                createAt: new Date()
            },
            ttl: option.ttl || this.settings.authentication.ttl || 1000 * 60 * 60 * 24 * 7
        }, function (err, id) {
            if (err) {
                callback(err);
                return;
            }
            callback(null, id);
        });
    };
    AuthenticationService.prototype.deleteToken = function (option, callback) {
        this.services.TTLSessionStorage.deleteTTLSession({
            id: option.token
        }, (callback));
    };
    AuthenticationService.prototype.getTokenInfo = function (option, callback) {
        this.services.TTLSessionStorage.getTTLSessionDataById({ id: option.token }, function (err, data) {
            callback(err, data);
        });
    };
    AuthenticationService.prototype.initialize = function (done) {
        this.authenticationCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.authenticationCollection.createIndex({
            guid: 1
        }, {
            unique: true
        });
        this.authenticationCollection.createIndex({
            accountId: 1,
            type: 1
        }, {
            unique: true
        });
        for (var name_1 in this.definitions) {
            var def = this.definitions[name_1];
            var keys = def.unique();
            if (keys) {
                for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                    var item = keys_1[_i];
                    this.authenticationCollection.createIndex(item, {
                        unique: true,
                        sparse: true
                    });
                }
            }
        }
        done();
    };
    AuthenticationService.prototype.define = function (definition) {
        if (this.definitions[definition.type]) {
            throw new Error("Definition for authentication " + definition.type + " already exists");
        }
        this.definitions[definition.type] = definition;
    };
    AuthenticationService.prototype.create = function (option, callback) {
        var auth = option.authentication;
        if (!auth || !auth.type || !auth.meta) {
            callback(new Errors.InvalidParameter("Invalid AuthenticationData provided"));
            return;
        }
        auth.createdAt = Date.now();
        var def = this.definitions[auth.type];
        if (!def) {
            callback(new Errors.NotFound("No definition found"));
            return;
        }
        try {
            var meta = def.sanitize(auth.meta);
            if (!meta) {
                callback(new Errors.InvalidParameter("Authentication can't pass format check"));
                return;
            }
            auth.meta = meta;
        }
        catch (e) {
            callback(new Errors.InvalidParameter(e.message));
            return;
        }
        auth.guid = def.guid(auth.meta);
        this.authenticationCollection.insert(auth, function (err) {
            if (err) {
                callback(new Errors.AlreadyExists());
                return;
            }
            callback(null, auth);
        });
    };
    AuthenticationService.prototype.authenticate = function (option, callback) {
        var auth = option.authentication;
        try {
            var def_1 = this.getDefinition(auth.type);
            var query = def_1.authenticateCriteria(auth.meta);
            if (!query) {
                callback(new Errors.InvalidParameter("Fail to generate authentication query"));
                return;
            }
            this.authenticationCollection.findOne(query, function (err, obj) {
                if (err) {
                    console.error(err);
                    callback(new Errors.NotFound("Database return error"));
                    return;
                }
                if (!obj) {
                    callback(new Errors.NotFound);
                    return;
                }
                delete obj["_id"];
                obj.meta = def_1.clearSensitiveInformation(obj.meta);
                callback(null, obj);
            });
        }
        catch (e) {
            callback(new Errors.InvalidParameter(e.message));
            return;
        }
    };
    AuthenticationService.prototype.get = function (option, callback) {
        var auth = option.authentication;
        try {
            var def_2 = this.getDefinition(auth.type);
            var query = def_2.queryCriteria(auth.meta);
            if (!query) {
                callback(new Errors.InvalidParameter("Fail to generate authentication query"));
                return;
            }
            this.authenticationCollection.findOne(query, function (err, obj) {
                if (err) {
                    console.error(err);
                    callback(new Errors.NotFound("Database return error"));
                    return;
                }
                if (!obj) {
                    callback(new Errors.NotFound);
                    return;
                }
                delete obj["_id"];
                obj.meta = def_2.clearSensitiveInformation(obj.meta);
                callback(null, obj);
            });
        }
        catch (e) {
            callback(new Errors.InvalidParameter(e.message));
            return;
        }
    };
    AuthenticationService.prototype.update = function (option, callback) {
        var auth = option.authentication;
        var def = this.sanitizeAndGetDefinition(auth);
        var guid = auth.guid;
        var updates = {
            $set: {}
        };
        for (var name_2 in auth.meta) {
            updates["$set"]["meta." + name_2] = auth.meta[name_2];
        }
        if (auth.accountId) {
            updates["$set"]["accountId"] = auth.accountId;
        }
        this.authenticationCollection.findOneAndUpdate({
            guid: guid
        }, updates, function (err, result) {
            if (err || result.ok !== 1) {
                callback(new Errors.NotFound);
                return;
            }
            callback(null, auth);
        });
    };
    AuthenticationService.prototype.findByAccountId = function (accountId, callback) {
        var _this = this;
        var cursor = this.authenticationCollection.find({
            accountId: accountId
        });
        //@ts-ignore: typescript generic bug
        cursor.toArray(function (err, objs) {
            if (err || !objs) {
                callback(new Errors.NotFound);
                return;
            }
            objs.map(function (obj) {
                var def = _this.getDefinition(obj.type);
                if (def) {
                    obj.meta = def.sanitize(obj.meta);
                    obj.meta = def.clearSensitiveInformation(obj.meta);
                }
                return obj;
            });
            callback(null, objs);
        });
    };
    AuthenticationService.prototype.sanitizeAndGetDefinition = function (auth) {
        if (!auth || !auth.type) {
            throw new Errors.InvalidParameter();
        }
        var def = this.getDefinition(auth.type);
        if (!def) {
            throw new Errors.NotFound("Unknown authentication type");
        }
        var meta = def.sanitize(auth.meta);
        if (!meta) {
            throw new Errors.InvalidParameter("Authentication meta fail to pass authenticaion sanitization");
        }
        auth.meta = meta;
        return def;
    };
    AuthenticationService.prototype.getDefinition = function (type) {
        if (!type)
            return null;
        return this.definitions[type] || null;
    };
    return AuthenticationService;
}(service_1.Service));
exports.AuthenticationService = AuthenticationService;
var AuthenticationDefinition = /** @class */ (function () {
    function AuthenticationDefinition() {
    }
    AuthenticationDefinition.prototype.abouts = function (meta, fields) {
        var v = {};
        for (var _i = 0, fields_1 = fields; _i < fields_1.length; _i++) {
            var field = fields_1[_i];
            v[field] = this.about(meta, field);
        }
        return v;
    };
    return AuthenticationDefinition;
}());
exports.AuthenticationDefinition = AuthenticationDefinition;
var CommonAuthenticationDefinition = /** @class */ (function (_super) {
    __extends(CommonAuthenticationDefinition, _super);
    function CommonAuthenticationDefinition(option) {
        if (option === void 0) { option = {}; }
        var _this = _super.call(this) || this;
        _this.option = option;
        _this.type = "common";
        _this.option = Leaf.Util.buildOption({
            salt: "e7ac4793242f40ec",
            maxNameLength: 64,
            minNameLength: 3,
            maxEmailLength: 128,
            minEmailLength: 6,
            minPasswordLength: 3,
            maxPasswordLength: 1024,
            nameValidator: /^[0-9a-z_]+$/i,
            emailValidator: /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
            passwordChecker: function (o) { return true; },
            requireActivation: false
        }, _this.option);
        return _this;
    }
    CommonAuthenticationDefinition.prototype.sanitize = function (meta) {
        var copy = __assign({}, meta);
        for (var _i = 0, _a = ["name", "email"]; _i < _a.length; _i++) {
            var field = _a[_i];
            if (!meta[field])
                throw new Errors.InvalidParameter("Missing " + field);
        }
        if (!meta.displayName) {
            meta.displayName = meta.name;
        }
        meta.name = meta.name.toLowerCase();
        if (meta.name.length > this.option.maxNameLength) {
            throw new Errors.InvalidParameter("Name length exceed max length " + this.option.maxNameLength);
        }
        if (meta.name.length < this.option.minNameLength) {
            throw new Errors.InvalidParameter("Name length exceed min length " + this.option.minNameLength);
        }
        if (!this.option.nameValidator.test(meta.name)) {
            throw new Errors.InvalidParameter("Invalid name, name can only contains 0-9, a-z and _");
        }
        if (meta.email.length > this.option.maxEmailLength) {
            throw new Errors.InvalidParameter("Email length exceed max length " + this.option.maxEmailLength);
        }
        if (meta.email.length < this.option.minEmailLength) {
            throw new Errors.InvalidParameter("Email length exceed min length " + this.option.minEmailLength);
        }
        if (!this.option.emailValidator.test(meta.email)) {
            throw new Errors.InvalidParameter("Invalid email formation");
        }
        if (meta.password) {
            if (meta.password.length > this.option.maxPasswordLength) {
                throw new Errors.InvalidParameter("Password length exceed max length " + this.option.maxPasswordLength);
            }
            if (meta.password.length < this.option.minPasswordLength) {
                throw new Errors.InvalidParameter("Password length exceed min length " + this.option.minPasswordLength);
            }
            if (!this.option.passwordChecker(meta.password)) {
                throw new Errors.InvalidParameter("Password too simple");
            }
        }
        if (meta.password) {
            delete copy["password"];
            copy.passwordHash = this.hash(meta.password);
        }
        if (!this.option.requireActivation)
            [
                copy.active = true
            ];
        return copy;
    };
    CommonAuthenticationDefinition.prototype.queryCriteria = function (meta) {
        var hash;
        if (!meta.name && !meta.email) {
            throw new Errors.InvalidParameter("No valid accountnamd and email provided");
        }
        var query = {};
        if (meta.email) {
            query = __assign({}, query, { "meta.email": meta.email });
        }
        else if (meta.name) {
            query = __assign({}, query, { "meta.name": meta.name });
        }
        return query;
    };
    CommonAuthenticationDefinition.prototype.authenticateCriteria = function (meta) {
        var hash;
        if (meta.password) {
            hash = this.hash(meta.password);
        }
        else if (meta.passwordHash) {
            hash = meta.passwordHash;
        }
        else {
            throw new Errors.InvalidParameter("No valid password/password has privided");
        }
        if (!meta.name && !meta.email) {
            throw new Errors.InvalidParameter("No valid accountnamd and email provided");
        }
        var query = {
            "meta.passwordHash": hash
        };
        if (meta.email) {
            query = __assign({}, query, { "meta.email": meta.email });
        }
        else if (meta.name) {
            query = __assign({}, query, { "meta.name": meta.name });
        }
        return query;
    };
    CommonAuthenticationDefinition.prototype.props = function (meta) {
        return {
            email: meta.email,
            name: meta.name
        };
    };
    CommonAuthenticationDefinition.prototype.clearSensitiveInformation = function (meta) {
        var copy = __assign({}, meta);
        delete copy.password;
        delete copy.passwordHash;
        return copy;
    };
    CommonAuthenticationDefinition.prototype.guid = function (meta) {
        return crypto.createHash("sha1").update(meta.name + meta.email).update(this.option.salt).digest("hex");
    };
    CommonAuthenticationDefinition.prototype.unique = function () {
        return [{
                "meta.email": 1
            }, {
                "meta.name": 1
            }];
    };
    CommonAuthenticationDefinition.prototype.about = function (meta, field) {
        if (field === "displayName")
            return meta.displayName || meta.name;
        if (field === "email")
            return meta.email;
        if (field === "name")
            return meta.name;
        return null;
    };
    CommonAuthenticationDefinition.prototype.hash = function (raw) {
        // 2 level hash creates the possibility that client don't give password to server.
        // Only the sha1 content are transfered
        var h1 = crypto.createHash("sha1").update(raw).digest("hex");
        var h2 = crypto.createHash("sha1").update(h1).update(this.option.salt).digest("hex");
        return h2;
    };
    return CommonAuthenticationDefinition;
}(AuthenticationDefinition));
exports.CommonAuthenticationDefinition = CommonAuthenticationDefinition;
//# sourceMappingURL=authenticationService.js.map