"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var Leaf = require("leaf-ts");
var crypto = require("crypto");
var fs = require("fs");
var fsExtra = require("fs-extra");
var pathModule = require("path");
var uuid = require("uuid");
exports.Errors = Leaf.ErrorDoc.build({
    HashError: {},
    UnknowError: {},
    NotFound: {}
});
var BlobService = /** @class */ (function (_super) {
    __extends(BlobService, _super);
    function BlobService(settings) {
        var _this = _super.call(this, {
            blob: {
                route: "/blob",
                storage: "/tmp/blob",
            }
        }, settings) || this;
        _this.name = "BlobService";
        _this.dependencies = ["MongodbService", "ExpressService"];
        _this.collectionName = "blob";
        _this.accessAuth = function (req, res, next) {
            next();
        };
        return _this;
    }
    BlobService.prototype.initialize = function (done) {
        var _this = this;
        this.blobCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.blobCollection.createIndex({
            id: 1
        });
        this.blobCollection.createIndex({
            hash: 1
        });
        fsExtra.ensureDirSync(this.settings.blob.storage);
        // ensure
        this.services.ExpressService.server.post(this.settings.blob.route + "/:blobId/GET", function (req, res) {
            _this.accessAuth(req, res, function () {
                _this.getBlob({
                    id: req.params.blobId
                }, function (err, blob) {
                    if (err) {
                        res.json({
                            error: new exports.Errors.NotFound(),
                            state: false
                        });
                        return;
                    }
                    res.json({
                        state: true,
                        data: blob
                    });
                });
            });
        });
        var route = this.settings.blob.route + "/:blobId/:anything";
        this.services.ExpressService.server.get(route, this.handleRouteDownload.bind(this));
        done();
    };
    BlobService.prototype.setAccessAuth = function (handler) {
        this.accessAuth = handler;
    };
    BlobService.prototype.handleRouteDownload = function (req, res) {
        var _this = this;
        this.accessAuth(req, res, function () {
            var id = req.params.blobId;
            console.error(id);
            _this.getBlob({ id: id }, function (err, blob) {
                if (err || blob == null) {
                    res.status(404).end("404");
                    return;
                }
                var path = _this.createPathByHash(blob.hash);
                res.sendFile(path);
            });
        });
    };
    BlobService.prototype.createBlob = function (option, callback) {
        var _this = this;
        this.stat(option.path, function (err, stat) {
            if (err) {
                callback(err);
                return;
            }
            var ext = pathModule.extname(option.name);
            var id = uuid.v4();
            var blob = {
                name: option.name,
                ext: ext,
                id: id,
                hash: stat.hash,
                size: stat.size,
                urls: [{
                        type: "local",
                        value: "/blob/" + id + "/" + option.name
                    }]
            };
            var targetPath = pathModule.join(_this.settings.blob.storage, blob.hash);
            _this.blobCollection.findOne({
                hash: blob.hash
            }, function (err, one) {
                if (one) {
                    callback(null, one);
                    return;
                }
                fs.link(option.path, targetPath, function (err) {
                    if (err) {
                        callback(new exports.Errors.UnknowError("Fail to hardlink"));
                        return;
                    }
                    _this.blobCollection.insert(blob, function (err, result) {
                        if (err) {
                            if (err.code == 10010) {
                                _this.blobCollection.findOne(blob, function (err, result) {
                                    delete result["_id"];
                                    callback(err, result);
                                });
                            }
                            else {
                                callback(new exports.Errors.UnknowError("Fail to update database", { via: err }));
                            }
                            return;
                        }
                    });
                    callback(null, blob);
                });
            });
        });
    };
    BlobService.prototype.stat = function (path, callback) {
        var hash = crypto.createHash("md5");
        var stream = fs.createReadStream(path);
        var size = 0;
        stream.on("data", function (data) {
            size += data.length;
            hash.update(data);
        });
        stream.on("error", function (err) {
            stream.destroy();
            callback(new exports.Errors.HashError("Fil to read disk file", { via: err }));
        });
        stream.on("end", function () {
            var hex = hash.digest("hex");
            callback(null, {
                hash: hex,
                size: size
            });
        });
    };
    BlobService.prototype.createPathByHash = function (hash) {
        var path = this.settings.blob.storage + "/" + hash;
        return path;
    };
    BlobService.prototype.getBlob = function (option, callback) {
        this.blobCollection.findOne({ id: option.id }, function (err, blob) {
            if (err || !blob) {
                callback(new exports.Errors.NotFound("not fond in mongodb"));
                return;
            }
            callback(null, blob);
        });
    };
    return BlobService;
}(service_1.Service));
exports.BlobService = BlobService;
//# sourceMappingURL=blobService.js.map