"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var ServiceSpec = require("../spec/service");
var async = require("async");
var GracefulShutdownService = /** @class */ (function (_super) {
    __extends(GracefulShutdownService, _super);
    function GracefulShutdownService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.exitHandlers = [];
        _this.exitHandler = _this.onExit.bind(_this);
        return _this;
    }
    GracefulShutdownService.prototype.initialize = function () {
        process.on("exit", this.exitHandler);
        process.on("SIGINT", this.exitHandler);
    };
    GracefulShutdownService.prototype.graceful = function (option, callback) {
        this.exitHandlers.push({
            name: option.name,
            timeout: option.timeout || this.settings.gracefulShutdown.timeout,
            handler: callback
        });
    };
    GracefulShutdownService.prototype.onExit = function () {
        async.forEachSeries(this.exitHandlers, function (todo, done) {
            var timer = setTimeout(function () {
                console.error("Graceful shutdown timeout for \"" + todo.name + "\"");
                console.error("Skip it");
                done();
                done = function () { };
            }, todo.timeout);
            todo.handler(function () {
                done();
                clearTimeout(timer);
                done = function () { };
            });
        }, function () {
            process.exit(0);
        });
    };
    return GracefulShutdownService;
}(ServiceSpec.GracefulShutdownService));
exports.GracefulShutdownService = GracefulShutdownService;
//# sourceMappingURL=gracefulShutdownService.js.map