"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.build({
    UnknownError: {},
    NotFound: {},
    InvalidParameter: {},
});
var IncrementalIdService = /** @class */ (function (_super) {
    __extends(IncrementalIdService, _super);
    function IncrementalIdService() {
        var _this = _super.call(this, {}, {}) || this;
        _this.name = "IncrementalIdService";
        _this.dependencies = ["MongodbService"];
        _this.collectionName = "IncrementalIdService";
        _this.defaultOffset = 1;
        _this.availableItems = {};
        return _this;
    }
    IncrementalIdService.prototype.initialize = function () {
        this.incrementalIdCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.incrementalIdCollection.createIndex({
            name: 1
        }, {
            unique: true
        });
    };
    IncrementalIdService.prototype.ensure = function (option, callback) {
        var _this = this;
        if (!option.name) {
            callback(new exports.Errors.InvalidParameter);
            return;
        }
        this.incrementalIdCollection.insertOne({
            name: option.name,
            offset: option.offset | this.defaultOffset
        }, function (err) {
            if (err && err["code"] !== 11000) {
                callback(new exports.Errors.UnknownError("Unknown error when insert base document", { via: err }));
                return;
            }
            _this.availableItems[option.name] = true;
            callback(null);
        });
    };
    IncrementalIdService.prototype.offset = function (option, callback) {
        if (!option.name) {
            callback(new exports.Errors.InvalidParameter);
            return;
        }
        this.incrementalIdCollection.findOneAndUpdate({
            name: option.name,
        }, {
            $set: {
                offset: option.offset
            }
        }, function (err) {
            callback(null);
        });
    };
    IncrementalIdService.prototype.next = function (option, callback) {
        if (!option.name) {
            callback(new exports.Errors.InvalidParameter);
            return;
        }
        if (!this.availableItems[option.name]) {
            callback(new exports.Errors.NotFound());
            return;
        }
        var count = option.count || 1;
        if (count < 0)
            count = 1;
        this.incrementalIdCollection.findOneAndUpdate({
            name: option.name
        }, {
            $inc: {
                offset: count
            }
        }, function (err, result) {
            if (err) {
                callback(err);
                return;
            }
            callback(null, result.value.offset);
        });
    };
    return IncrementalIdService;
}(service_1.Service));
exports.IncrementalIdService = IncrementalIdService;
//# sourceMappingURL=incrementalIdService.js.map