import * as ServiceSpec from "../spec/service"
import * as mongodb from "mongodb"
import { Errors } from "../spec/errors"

export class KeyValueStorage extends ServiceSpec.KeyValueStorage {
    collection: mongodb.Collection<{
        key: string,
        value: any
    }>
    initialize(done) {
        this.collection = this.services.MongodbService.db.collection(this.settings.kvs.collection)
        this.collection.createIndex({
            key: 1
        }, {
                unique: true
            }, done)
    }
    set<T>(k: string, v: T, callback?: Callback<null>) {
        this.collection.findOneAndUpdate({ key: k }, {
            key: k,
            value: v
        }, {
                upsert: true
            }, (err, result) => {
                callback(err)
            })

    }
    get<T>(k: string, callback: Callback<T>) {
        this.collection.findOne({
            key: k
        }, (err, value) => {
            if (err || !value) {
                callback(new Errors.NotFound())
                return
            }
            callback(null, value.value as any as T)
        })
    }
}
