"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.create()
    .define("InvalidLegacyUsername")
    .define("LegacyUsernameTooShort")
    .define("LegacyUsernameTooLong")
    .define("InvalidLegacyUsername")
    .define("PasswordTooShort")
    .define("PasswordTooLong")
    .define("InvalidPassword")
    .define("InvalidEmail")
    .define("EmailTooLong")
    .define("InvalidDisplayName")
    .define("DisplayNameTooShort")
    .define("DisplayNameTooLong")
    .define("LegacyUsernameRequired")
    .define("PasswordRequired")
    .define("EmailRequired")
    .generate();
var LegacyUserDefinitionService = /** @class */ (function (_super) {
    __extends(LegacyUserDefinitionService, _super);
    function LegacyUserDefinitionService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.name = "LegacyUserDefinitionService";
        _this.LegacyUserPartialValidator = GenerateLegacyUserValidator();
        _this.LegacyUserValidator = _this.LegacyUserPartialValidator.clone()
            .field("legacyUsername").exists().error(new exports.Errors.LegacyUsernameRequired)
            .field("email").exists().error(new exports.Errors.EmailRequired)
            .field("password").exists().error(new exports.Errors.PasswordRequired);
        return _this;
    }
    return LegacyUserDefinitionService;
}(service_1.Service));
exports.LegacyUserDefinitionService = LegacyUserDefinitionService;
exports.DefaultLegacyUserValidationConfig = {
    minLegacyUsernameLength: 3,
    maxLegacyUsernameLength: 32,
    minDisplayNameLength: 3,
    maxDisplayNameLength: 32,
    minPasswordLength: 8,
    maxPasswordLength: 256,
    maxEmailLength: 128
};
function GenerateLegacyUserValidator(config) {
    if (config === void 0) { config = exports.DefaultLegacyUserValidationConfig; }
    return Leaf.Validator.create()
        .field("legacyUsername")
        .string().error(new exports.Errors.InvalidLegacyUsername)
        .gt(config.minLegacyUsernameLength - 1).error(new exports.Errors.LegacyUsernameTooShort)
        .lt(config.maxLegacyUsernameLength + 1).error(new exports.Errors.LegacyUsernameTooLong)
        .match(/^[0-9a-z_]+$/i).error(new exports.Errors.InvalidLegacyUsername)
        .field("displayName")
        .string().error(new exports.Errors.InvalidDisplayName)
        .gt(config.minDisplayNameLength - 1).error(new exports.Errors.DisplayNameTooShort)
        .lt(config.maxDisplayNameLength + 1).error(new exports.Errors.DisplayNameTooLong)
        .field("password")
        .string().error(new exports.Errors.InvalidPassword)
        .gt(config.minPasswordLength - 1).error(new exports.Errors.PasswordTooShort)
        .lt(config.maxPasswordLength + 1).error(new exports.Errors.PasswordTooLong)
        .field("email")
        .lt(config.maxEmailLength + 1).error(new exports.Errors.EmailTooLong)
        .match(/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/)
        .readonly();
}
exports.GenerateLegacyUserValidator = GenerateLegacyUserValidator;
//# sourceMappingURL=legacyUserDefinitionService.js.map