import { Service } from "../service";
import { LegacyUserDefinitionService, LegacyUser } from "./legacyUserDefinitionService";
import { MongodbService } from "./mongodbService";
import * as mongodb from "mongodb";
import * as Leaf from "leaf-ts";
export declare type Callback<T> = (err?: Error, result?: T) => void;
export declare const Errors: Leaf.ErrorsOf<{
    UnknownError: {};
    DBError: {};
    AlreadyExists: {};
    InvalidParameter: {};
    LegacyUsernameAlreadyExists: {};
    EmailAlreadyExists: {};
}>;
export declare class LegacyUserService<TLegacyUser extends LegacyUser = LegacyUser> extends Service {
    readonly name: string;
    readonly dependencies: any;
    readonly collectionName: string;
    readonly Errors: Leaf.ErrorsOf<{
        UnknownError: {};
        DBError: {};
        AlreadyExists: {};
        InvalidParameter: {};
        LegacyUsernameAlreadyExists: {};
        EmailAlreadyExists: {};
    }>;
    constructor(config?: {
        legacyUser?: {
            salt?: string;
            blacklist?: string[];
        };
    });
    private blacklist;
    private salt;
    services: {
        MongodbService: MongodbService;
        LegacyUserDefinitionService: LegacyUserDefinitionService<TLegacyUser>;
    };
    legacyUserCollection: mongodb.Collection;
    initialize(): void;
    private createPasswordHashString(rawPassword);
    private translateMongoId(doc);
    toPublicLegacyUser(legacyUser: TLegacyUser): TLegacyUser;
    getLegacyUserById(id: string, callback: Callback<TLegacyUser>): void;
    getLegacyUserByLegacyUsername(legacyUsername: string, callback: Callback<TLegacyUser>): void;
    getLegacyUserByEmail(email: string, callback: Callback<TLegacyUser>): void;
    getLegacyUserByLogin(option: {
        login: string;
        password: string;
    }, callback: Callback<TLegacyUser>): void;
    createLegacyUser(partialLegacyUser: Partial<TLegacyUser>, callback: Callback<TLegacyUser>): void;
    updateLegacyUserPasswordByLegacyUsername(option: {
        legacyUsername: string;
        password: string;
    }, callback: Callback<TLegacyUser>): void;
    updateLegacyUserPasswordById(option: {
        id: string;
        password: string;
    }, callback: Callback<TLegacyUser>): void;
}
