"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var mongodb = require("mongodb");
var crypto = require("crypto");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.create()
    .define("UnknownError")
    .define("DBError")
    .define("AlreadyExists")
    .define("LegacyUsernameAlreadyExists")
    .define("EmailAlreadyExists")
    .define("InvalidParameter")
    .generate();
function clearLegacyUserPassword(obj, name, descriptor) {
    var fn = descriptor.value;
    descriptor.value = function (option, callback) {
        fn.call(this, option, function (err, value) {
            if (value instanceof Array) {
                value = value.forEach(function (item) {
                    delete item.passwordHash;
                });
            }
            else if (value) {
                delete value.passwordHash;
            }
            callback(err, value);
        });
    };
}
var LegacyUserService = /** @class */ (function (_super) {
    __extends(LegacyUserService, _super);
    function LegacyUserService(config) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this, {}, {}) || this;
        _this.name = "LegacyUserService";
        _this.dependencies = ["MongodbService", "LegacyUserDefinitionService"];
        _this.collectionName = "legacyUser";
        _this.Errors = exports.Errors;
        _this.blacklist = [];
        _this.salt = "default-legacyUser-secret-salt";
        if (config.legacyUser) {
            if (config.legacyUser.salt) {
                _this.salt = config.legacyUser.salt;
            }
            if (config.legacyUser.blacklist) {
                _this.blacklist = config.legacyUser.blacklist;
            }
        }
        return _this;
    }
    LegacyUserService.prototype.initialize = function () {
        this.salt;
        this.legacyUserCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.legacyUserCollection.createIndex({
            legacyUsername: 1
        }, {
            unique: true
        });
        this.legacyUserCollection.createIndex({
            email: 1
        }, {
            unique: true
        });
        this.legacyUserCollection.createIndex({
            passwordHash: 1
        });
    };
    LegacyUserService.prototype.createPasswordHashString = function (rawPassword) {
        var level1 = crypto.createHash("sha256").update(rawPassword + this.salt).digest('hex');
        var level2 = crypto.createHash("sha512").update(level1 + this.salt).digest('hex');
        return level2;
    };
    LegacyUserService.prototype.translateMongoId = function (doc) {
        if (!doc)
            return null;
        var id = doc._id;
        delete doc._id;
        doc.id = id.toString();
        return doc;
    };
    LegacyUserService.prototype.toPublicLegacyUser = function (legacyUser) {
        delete legacyUser.passwordHash;
        delete legacyUser.password;
        return legacyUser;
    };
    LegacyUserService.prototype.getLegacyUserById = function (id, callback) {
        var _this = this;
        var _id = new mongodb.ObjectID(id);
        this.legacyUserCollection.findOne({ _id: _id }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var legacyUser = _this.translateMongoId(doc);
            callback(null, legacyUser);
        });
    };
    LegacyUserService.prototype.getLegacyUserByLegacyUsername = function (legacyUsername, callback) {
        var _this = this;
        legacyUsername = legacyUsername.toLowerCase();
        this.legacyUserCollection.findOne({ legacyUsername: legacyUsername }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var legacyUser = _this.translateMongoId(doc);
            callback(null, legacyUser);
        });
    };
    LegacyUserService.prototype.getLegacyUserByEmail = function (email, callback) {
        var _this = this;
        email = email.toLowerCase();
        this.legacyUserCollection.findOne({ email: email }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var legacyUser = _this.translateMongoId(doc);
            callback(null, legacyUser);
        });
    };
    LegacyUserService.prototype.getLegacyUserByLogin = function (option, callback) {
        var _this = this;
        var login = option.login.toLowerCase();
        var passwordHash = this.createPasswordHashString(option.password);
        this.legacyUserCollection.findOne({ passwordHash: passwordHash, $or: [{ legacyUsername: login }, { email: login }] }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var legacyUser = _this.translateMongoId(doc);
            callback(null, legacyUser);
        });
    };
    LegacyUserService.prototype.createLegacyUser = function (partialLegacyUser, callback) {
        var _this = this;
        if (this.blacklist.indexOf(partialLegacyUser.legacyUsername) >= 0) {
            callback(new exports.Errors.InvalidParameter("Invalid legacyUsername"));
            return;
        }
        if (!partialLegacyUser.legacyUsername || !partialLegacyUser.email) {
            callback(new exports.Errors.InvalidParameter("Empty legacyUsername or email"));
            return;
        }
        // use legacyUsername as display name by default
        if (!partialLegacyUser.displayName) {
            partialLegacyUser.displayName = partialLegacyUser.legacyUsername;
        }
        // then convert legacyUsername to all lowercase
        partialLegacyUser.legacyUsername = partialLegacyUser.legacyUsername.toLocaleLowerCase();
        partialLegacyUser.email = partialLegacyUser.email.toLowerCase();
        try {
        }
        catch (error) {
            callback(error);
            return;
        }
        try {
            this.services.LegacyUserDefinitionService.LegacyUserValidator.check(partialLegacyUser);
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var passwordHash = this.createPasswordHashString(partialLegacyUser.password);
        var legacyUser = {
            legacyUsername: partialLegacyUser.legacyUsername,
            displayName: partialLegacyUser.displayName,
            passwordHash: passwordHash,
            email: partialLegacyUser.email,
            crteateAt: Date.now()
        };
        this.getLegacyUserByLegacyUsername(legacyUser.legacyUsername, function (error, doc) {
            if (doc) {
                callback(new exports.Errors.LegacyUsernameAlreadyExists());
                return;
            }
            _this.getLegacyUserByEmail(legacyUser.email, function (error, doc) {
                if (doc) {
                    callback(new exports.Errors.EmailAlreadyExists());
                    return;
                }
                _this.legacyUserCollection.insert(legacyUser, function (error, doc) {
                    if (error) {
                        // on a edge case that 2 legacyUser try to use a same legacyUsername and something like that
                        callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                        return;
                    }
                    var legacyUser = _this.translateMongoId(doc.ops[0]);
                    callback(null, legacyUser);
                });
            });
        });
    };
    LegacyUserService.prototype.updateLegacyUserPasswordByLegacyUsername = function (option, callback) {
        var _this = this;
        try {
            this.services.LegacyUserDefinitionService.LegacyUserValidator.check({ password: option.password });
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var legacyUsername = option.legacyUsername.toLowerCase();
        var passwordHash = this.createPasswordHashString(option.password);
        this.legacyUserCollection.findOneAndUpdate({ legacyUsername: legacyUsername }, { $set: { passwordHash: passwordHash } }, { returnOriginal: false }, function (error, doc) {
            if (error) {
                callback(error);
                return;
            }
            var legacyUser = _this.translateMongoId(doc.value);
            callback(null, legacyUser);
        });
    };
    LegacyUserService.prototype.updateLegacyUserPasswordById = function (option, callback) {
        var _this = this;
        var _id = new mongodb.ObjectID(option.id);
        try {
            this.services.LegacyUserDefinitionService.LegacyUserValidator.check({ password: option.password });
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var passwordHash = this.createPasswordHashString(option.password);
        this.legacyUserCollection.findOneAndUpdate({ _id: _id }, { $set: { passwordHash: passwordHash } }, { returnOriginal: false }, function (error, doc) {
            if (error) {
                callback(error);
                return;
            }
            var legacyUser = _this.translateMongoId(doc.value);
            callback(null, legacyUser);
        });
    };
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "getLegacyUserById", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "getLegacyUserByLegacyUsername", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "getLegacyUserByEmail", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "getLegacyUserByLogin", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "createLegacyUser", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "updateLegacyUserPasswordByLegacyUsername", null);
    __decorate([
        clearLegacyUserPassword
    ], LegacyUserService.prototype, "updateLegacyUserPasswordById", null);
    return LegacyUserService;
}(service_1.Service));
exports.LegacyUserService = LegacyUserService;
//# sourceMappingURL=legacyUserService.js.map