"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var async = require("async");
var mongodb = require("mongodb");
var MongodbService = /** @class */ (function (_super) {
    __extends(MongodbService, _super);
    function MongodbService(settings) {
        var _this = _super.call(this, {
            database: {
                port: 27017,
                host: "localhost",
                name: "test"
            }
        }, settings) || this;
        _this.name = "MongodbService";
        return _this;
    }
    MongodbService.prototype.initialize = function (done) {
        var _this = this;
        var _a = this.settings.database, host = _a.host, port = _a.port, name = _a.name;
        console.log("Connecting to database", "mongodb://" + host + ":" + port + "/" + name);
        mongodb.MongoClient.connect("mongodb://" + host + ":" + port, function (err, client) {
            _this.db = client.db(name);
            done(err);
        });
    };
    MongodbService.prototype.asBuiltIn = function () {
        var _a = this.settings.database, host = _a.host, port = _a.port, name = _a.name;
        console.log("MongodbService using mongo://" + host + ":" + port + "/" + name);
    };
    MongodbService.prototype.createIndexBuilder = function () {
        return new IndexBuilder();
    };
    return MongodbService;
}(service_1.Service));
exports.MongodbService = MongodbService;
var IndexBuilder = /** @class */ (function () {
    function IndexBuilder() {
        this.cols = [];
        this.current = null;
    }
    IndexBuilder.prototype.collection = function (collection) {
        for (var _i = 0, _a = this.cols; _i < _a.length; _i++) {
            var col = _a[_i];
            if (col.collection === collection) {
                this.current = col;
                return;
            }
        }
        this.current = {
            collection: collection,
            indexes: []
        };
        this.cols.push(this.current);
        return this;
    };
    IndexBuilder.prototype.index = function (key, option) {
        if (option === void 0) { option = {}; }
        if (!this.current) {
            throw new Error("Set index before specify collection, please call IndexBuilder.collection first.");
        }
        if (typeof key == "string") {
            key = (_a = {},
                _a[key] = 1,
                _a);
        }
        if (key instanceof Array) {
            var _k = {};
            for (var _i = 0, key_1 = key; _i < key_1.length; _i++) {
                var v = key_1[_i];
                if (v[0] == "-") {
                    _k[v.slice(1)] = -1;
                }
                else {
                    _k[v] = 1;
                }
            }
            key = _k;
        }
        this.current.indexes.push({
            key: key, option: option
        });
        return this;
        var _a;
    };
    IndexBuilder.prototype.lastIndex = function () {
        if (!this.current) {
            throw new Error("Should specify collection");
        }
        var info = this.current.indexes[this.current.indexes.length - 1];
        if (!info) {
            throw new Error("Should specify colelction and index");
        }
        return info;
    };
    IndexBuilder.prototype.unique = function () {
        var info = this.lastIndex();
        info.option.unique = true;
        return this;
    };
    IndexBuilder.prototype.nonunique = function () {
        var info = this.lastIndex();
        info.option.unique = false;
        return this;
    };
    IndexBuilder.prototype.sparse = function () {
        var info = this.lastIndex();
        info.option.sparse = true;
        return this;
    };
    IndexBuilder.prototype.nonsparse = function () {
        var info = this.lastIndex();
        info.option.sparse = false;
        return this;
    };
    IndexBuilder.prototype.apply = function (callback) {
        var cols = this.cols;
        async.eachSeries(cols, function (colInfo, done) {
            async.eachSeries(colInfo.indexes, function (data, done2) {
                if (!data.option)
                    data.option = {};
                console.warn("Indexing " + colInfo.collection.namespace + ":" + Object.keys(data.key).join("&") + " => " + (data.option.unique && "unique" || "none-unique") + " " + (data.option.sparse && "sparse" || ""));
                colInfo.collection.createIndex(data.key, {
                    unique: data.option.unique,
                    sparse: data.option.sparse
                }, done2);
            }, done);
        }, callback);
    };
    return IndexBuilder;
}());
exports.IndexBuilder = IndexBuilder;
//# sourceMappingURL=mongodbService.js.map