import { Service } from "../service";
import { MongodbService } from "./mongodbService";
import * as Leaf from "leaf-ts";
export declare type Callback<T = any, E extends Error = Error> = (e?: E, result?: T) => void;
export declare const Errors: {
    UnknownError: Leaf.CustomErrorConstructor<{}>;
    NotFound: Leaf.CustomErrorConstructor<{}>;
};
export declare class TTLSessionStorage extends Service {
    name: string;
    dependencies: string[];
    services: {
        MongodbService: MongodbService;
    };
    constructor();
    initialize(): void;
    defaultTTL: number;
    private collectionName;
    private ttlCollection;
    createTTLSession(option: {
        id?: string;
        data: any;
        ttl?: number;
    }, callback: Callback<string>): void;
    getOrCreateTTLSession<T>(option: {
        id: string;
        data: T;
        ttl: number;
    }, callback: Callback<{
        id: string;
        data: T;
        expire: Date;
    }>): void;
    deleteTTLSession(option: {
        id: string;
    }, callback: Callback<null>): void;
    updateTTLSession(option: {
        id: string;
        data: any;
        refreshTTL?: number;
    }, callback: Callback<TTLSessionData>): void;
    getTTLSessionDataById<T>(option: {
        id: string;
    }, callback: Callback<T>): void;
}
export interface TTLSessionData {
    id: string;
    data: any;
    expire: Date;
}
