"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var Leaf = require("leaf-ts");
var UUID = require("uuid");
exports.Errors = Leaf.ErrorDoc.build({
    UnknownError: {},
    NotFound: {},
});
var TTLSessionStorage = /** @class */ (function (_super) {
    __extends(TTLSessionStorage, _super);
    function TTLSessionStorage() {
        var _this = _super.call(this, null, null) || this;
        _this.name = "TTLSessionStorage";
        _this.dependencies = ["MongodbService"];
        _this.defaultTTL = 1000 * 60 * 24;
        _this.collectionName = "ttl";
        return _this;
    }
    TTLSessionStorage.prototype.initialize = function () {
        this.ttlCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.ttlCollection.createIndex({
            expire: 1
        }, {
            expireAfterSeconds: 0
        });
        this.ttlCollection.createIndex({
            id: 1
        }, {
            unique: true
        });
    };
    TTLSessionStorage.prototype.createTTLSession = function (option, callback) {
        var ttl = option.ttl || this.defaultTTL;
        var expire = new Date(Date.now() + ttl);
        var id = option.id || UUID.v4();
        var session = {
            id: id,
            data: option.data,
            expire: expire
        };
        this.ttlCollection.insertOne(session, function (err) {
            callback(err, id);
        });
    };
    TTLSessionStorage.prototype.updateOrCreateTTLSession = function (option, callback) {
        var ttl = option.ttl || this.defaultTTL;
        var updates = {};
        updates = {
            $setOnInsert: {
                id: option.id,
                expire: new Date(Date.now() + ttl)
            },
            $set: {
                data: option.data,
            }
        };
        // user specified ttl, means we should update the ttl as well
        if (option.ttl) {
            // have to $set and $setOnInsert can't be both exists
            delete updates["$setOnInsert"]["expire"];
            updates["$set"]["expire"] = new Date(Date.now() + option.ttl);
        }
        this.ttlCollection.findOneAndUpdate({
            id: option.id
        }, updates, {
            returnOriginal: false,
            upsert: true
        }, function (err, doc) {
            if (err) {
                callback(err);
                return;
            }
            callback(null, doc.value);
        });
    };
    TTLSessionStorage.prototype.getOrCreateTTLSession = function (option, callback) {
        throw new Error("Deprecated, please using touchOrCreateTTLSession or updateOrCreateTTLSession instead");
    };
    TTLSessionStorage.prototype.touchOrCreateTTLSession = function (option, callback) {
        var ttl = option.ttl || this.defaultTTL;
        var updates = {};
        // never update data using this function
        updates = {
            $setOnInsert: {
                id: option.id,
                data: option.data,
                expire: new Date(Date.now() + ttl),
            },
        };
        // updates expires
        // so using $set not $setOnInsert
        if (option.ttl) {
            // have to $set and $setOnInsert can't be both exists
            delete updates["$setOnInsert"]["expire"];
            updates["$set"] = {
                expire: new Date(Date.now() + option.ttl)
            };
        }
        this.ttlCollection.findOneAndUpdate({
            id: option.id
        }, updates, {
            returnOriginal: false,
            upsert: true
        }, function (err, doc) {
            if (err) {
                callback(err);
                return;
            }
            callback(null, doc.value);
        });
    };
    TTLSessionStorage.prototype.deleteTTLSession = function (option, callback) {
        this.ttlCollection.findOneAndDelete({
            id: option.id
        }, function (err) {
            callback();
        });
    };
    TTLSessionStorage.prototype.updateTTLSession = function (option, callback) {
        var updates = {
            data: option.data
        };
        if (typeof option.refreshTTL == "number") {
            updates["expire"] = new Date(Date.now() + option.refreshTTL);
        }
        this.ttlCollection.findOneAndUpdate({ id: option.id }, {
            $set: updates
        }, function (err, result) {
            if (err) {
                callback(err);
                return;
            }
            if (result.ok !== 1) {
                callback(new exports.Errors.UnknownError("Fail to find and replace TTLSession", { via: result.lastErrorObject }));
                return;
            }
            callback(null, result.value);
        });
    };
    TTLSessionStorage.prototype.getTTLSessionDataById = function (option, callback) {
        this.ttlCollection.findOne({ id: option.id }, function (err, result) {
            if (err) {
                callback(err);
                return;
            }
            if (!result) {
                callback(new exports.Errors.NotFound());
                return;
            }
            var data = result.data;
            callback(null, data);
        });
    };
    return TTLSessionStorage;
}(service_1.Service));
exports.TTLSessionStorage = TTLSessionStorage;
//# sourceMappingURL=ttlSessionStorage.js.map