"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Leaf = require("leaf-ts");
var service_1 = require("../lib/service");
var fsExtra = require("fs-extra");
var multer = require("multer");
var fs = require("fs");
var pathModule = require("path");
exports.Errors = Leaf.ErrorDoc.build({
    UploadError: {},
});
var UploadService = /** @class */ (function (_super) {
    __extends(UploadService, _super);
    function UploadService(settings) {
        var _this = _super.call(this, {
            upload: {
                temp: "/tmp/upload",
                route: "/upload",
                // default support all extnames
                acceptedExtnames: null,
                maxSizeInMB: 8
            }
        }, settings) || this;
        _this.name = "UploadService";
        _this.dependencies = ["ExpressService", "BlobService"];
        return _this;
    }
    UploadService.prototype.error = function (res, err) {
        res.json({
            state: false,
            error: err
        });
    };
    UploadService.prototype.success = function (res, data) {
        res.json({
            state: true,
            data: data
        });
    };
    UploadService.prototype.initialize = function () {
        var storageConfig = this.createMulterConfig();
        var multerMiddleware = multer({
            storage: storageConfig,
            limits: {
                fieldSize: this.settings.upload.maxSizeInMB
            }
        });
        this.services.ExpressService.server.post(this.settings.upload.route, this.createUploadHandler(multerMiddleware));
        fsExtra.ensureDirSync(this.settings.upload.temp);
    };
    UploadService.prototype.createUploadHandler = function (middleware) {
        var _this = this;
        var receive = middleware.single("file");
        return function (req, res) {
            receive(req, res, function (err) {
                if (err) {
                    res.status(400);
                    _this.error(res, new exports.Errors.UploadError("Lib error"));
                    return;
                }
                if (!req.file) {
                    res.status(400);
                    _this.error(res, new exports.Errors.UploadError("No file"));
                    return;
                }
                var file = req.file;
                var path = file.path;
                var originalName = file.originalname;
                fs.stat(path, function (err, stat) {
                    if (err) {
                        res.status(500);
                        _this.error(res, new exports.Errors.UploadError("Fail to find temp file", { via: err }));
                        return;
                    }
                    _this.services.BlobService.createBlob({ path: path, name: originalName }, function (err, blob) {
                        if (err) {
                            res.status(500);
                            _this.error(res, new exports.Errors.UploadError("Fail to create blob", { via: err }));
                            return;
                        }
                        _this.success(res, blob);
                    });
                });
            });
        };
    };
    UploadService.prototype.createMulterConfig = function () {
        var _this = this;
        var storage = multer.diskStorage({
            destination: function (req, file, cb) {
                cb(null, _this.settings.upload.temp);
            },
            filename: function (req, file, cb) {
                var fileFormat = (file.originalname).split('.');
                var ext = pathModule.extname(file.originalname);
                var whitelist = _this.settings.upload.acceptedExtnames;
                if (whitelist && whitelist.length > 0) {
                    if (whitelist.indexOf(ext) < 0) {
                        cb(new Error("Extname not supported"), null);
                        return;
                    }
                }
                if (!ext || ext.length > 8) {
                    ext = ".unknown";
                }
                var rand = Date.now() + Math.random().toString().slice(4, 12);
                cb(null, rand + ext);
            },
        });
        return storage;
    };
    return UploadService;
}(service_1.Service));
exports.UploadService = UploadService;
//# sourceMappingURL=uploadService.js.map