"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../lib/service");
var wxkit = require("wxkit");
var WechatService = /** @class */ (function (_super) {
    __extends(WechatService, _super);
    function WechatService(settings) {
        var _this = _super.call(this, {
            wechat: {
                appId: null,
                secret: null,
                domain: {
                    filename: null,
                    content: null
                },
                baseUrl: null
            }
        }, settings) || this;
        _this.settings = settings;
        _this.name = "WechatService";
        _this.dependencies = ["ExpressService", "RouteSessionService"];
        _this.befores = ["page.default"];
        _this.client = new wxkit.WechatClient({
            appId: _this.settings.wechat.appId,
            secret: _this.settings.wechat.secret
        });
        return _this;
    }
    WechatService.prototype.initialize = function () {
        var _this = this;
        this.services.ExpressService.server.post("/wechat/jssdk/signature/GET", function (req, res) {
            _this.client.createJSSDKConfig({
                url: req.body.url
            }, function (err, config) {
                if (err) {
                    res.json({
                        state: false,
                        error: err
                    });
                    return;
                }
                res.json({
                    state: true,
                    data: config
                });
            });
        });
        this.services.ExpressService.server.post("/wechat/user/profile/GET", function (req, res) {
            var callback = req.query.callback;
            // cached profile
            if (req.session.profile) {
                res.json({
                    state: true,
                    data: req.session.profile
                });
                return;
            }
            var openId = req.session.openId;
            var accessToken = req.session.accessToken;
            // get profile
            if (!openId || !accessToken) {
                res.json({
                    state: false,
                    error: "WeChatNotAuthorized"
                });
                return;
            }
            _this.client.getWebUserClient(function (err, client) {
                if (err) {
                    res.json({
                        state: false,
                        error: "ServerError"
                    });
                    return;
                }
                client.getUserProfile({
                    accessToken: accessToken, openId: openId
                }, function (err, profile) {
                    if (err) {
                        res.json({
                            state: false,
                            error: "WeChatProfileFailed"
                        });
                        return;
                    }
                    req.session.profile = profile;
                    req.session.save(function () { });
                    res.json({
                        state: true,
                        data: profile
                    });
                });
            });
        });
        this.services.ExpressService.server.get("/wechat/user/auth/REDIRECT", function (req, res) {
            var callback = req.query.callback;
            var scope = req.query.scope || "snsapi_userinfo";
            if (req.session.accessToken) {
                res.redirect(302, callback);
                return;
            }
            var backUrl = _this.settings.wechat.baseUrl + "wechat/oauth2/redirect/CALLBACK";
            _this.client.getWebUserClient(function (err, client) {
                if (err) {
                    res.send("Error");
                    return;
                }
                var url = client.createCodeRequestUrl({
                    backUrl: backUrl,
                    scope: scope,
                    state: callback
                });
                res.redirect(302, url);
            });
        });
        this.services.ExpressService.server.get("/wechat/oauth2/redirect/CALLBACK", function (req, res) {
            var code = req.query.code;
            var callback = req.query.state || "/";
            _this.client.getWebUserClient(function (err, client) {
                if (err) {
                    res.end("Error");
                    return;
                }
                if (!code) {
                    res.end("ERROR wx request");
                    return;
                }
                client.exchangeCodeForInformation({ code: code }, function (err, info) {
                    if (err) {
                        res.end(JSON.stringify(err));
                        return;
                    }
                    req.session.openId = info.openId;
                    req.session.accessToken = info.accessInfo.token;
                    req.session.save(function () {
                        res.redirect(302, callback);
                    });
                });
            });
        });
        if (this.settings.wechat && this.settings.wechat.domain) {
            this.services.ExpressService.server.get("/" + this.settings.wechat.domain.filename, function (req, res) {
                res.end(_this.settings.wechat.domain.content);
            });
        }
    };
    return WechatService;
}(service_1.Service));
exports.WechatService = WechatService;
//# sourceMappingURL=wechatService.js.map