import { ExpressService } from "./expressService"
import { Service } from "../lib/service"
import * as Leaf from "leaf-ts"
import * as wxkit from "wxkit"

export class WechatService extends Service {
    readonly name = "WechatService"
    readonly dependencies = ["ExpressService", "RouteSessionService"]
    constructor(public settings: WechatService.Settings) {
        super({
            wechat: {
                appId: null,
                secret: null,
                domain: {
                    filename: null,
                    content: null
                },
                baseUrl: null
            }
        }, settings)
    }
    services: { ExpressService: ExpressService }
    befores = ["page.default"]
    initialize() {
        this.services.ExpressService.server.post("/wechat/jssdk/signature/GET", (req, res) => {
            this.client.createJSSDKConfig({
                url: req.body.url
            }, (err, config) => {
                if (err) {
                    res.json({
                        state: false,
                        error: err
                    })
                    return
                }
                res.json({
                    state: true,
                    data: config
                })
            })
        })
        this.services.ExpressService.server.post("/wechat/user/profile/GET", (req, res) => {
            let callback = req.query.callback
            // cached profile
            if (req.session.profile) {
                res.json({
                    state: true,
                    data: req.session.profile
                })
                return
            }
            let openId = req.session.openId
            let accessToken = req.session.accessToken
            // get profile
            if (!openId || !accessToken) {
                res.json({
                    state: false,
                    error: "WeChatNotAuthorized"
                })
                return
            }
            this.client.getWebUserClient((err, client) => {
                if (err) {
                    res.json({
                        state: false,
                        error: "ServerError"
                    })
                    return
                }
                client.getUserProfile({
                    accessToken, openId
                }, (err, profile) => {
                    if (err) {
                        res.json({
                            state: false,
                            error: "WeChatProfileFailed"
                        })
                        return
                    }
                    req.session.profile = profile
                    req.session.save(() => { })
                    res.json({
                        state: true,
                        data: profile
                    })
                })

            })
        })
        this.services.ExpressService.server.get("/wechat/user/auth/REDIRECT", (req, res) => {
            let callback = req.query.callback
            let scope = req.query.scope || "snsapi_userinfo"
            if (req.session.accessToken) {
                res.redirect(302, callback)
                return
            }
            let backUrl = this.settings.wechat.baseUrl + `wechat/oauth2/redirect/CALLBACK`
            this.client.getWebUserClient((err, client) => {
                if (err) {
                    res.send("Error")
                    return
                }
                let url = client.createCodeRequestUrl({
                    backUrl,
                    scope,
                    state: callback
                })
                res.redirect(302, url)
            })
        })
        this.services.ExpressService.server.get("/wechat/oauth2/redirect/CALLBACK", (req, res) => {
            let code = req.query.code
            let callback = req.query.state || "/"
            this.client.getWebUserClient((err, client) => {
                if (err) {
                    res.end("Error")
                    return
                }
                if (!code) {
                    res.end("ERROR wx request")
                    return
                }
                client.exchangeCodeForInformation({ code }, (err, info) => {
                    if (err) {
                        res.end(JSON.stringify(err))
                        return
                    }
                    req.session.openId = info.openId
                    req.session.accessToken = info.accessInfo.token
                    req.session.save(() => {
                        res.redirect(302, callback)
                    })
                })
            })
        })
        if (this.settings.wechat && this.settings.wechat.domain) {
            this.services.ExpressService.server.get("/" + this.settings.wechat.domain.filename, (req, res) => {
                res.end(this.settings.wechat.domain.content)
            })
        }
    }
    client: wxkit.WechatClient = new wxkit.WechatClient({
        appId: this.settings.wechat.appId,
        secret: this.settings.wechat.secret
    })
}
