type Callback<T, E = Error> = (err?: E, data?: T) => void
type GUID = string

declare namespace GracefulShutdownService {
    type Settings = {
        gracefulShutdown: {
            // default timeout for a single job
            timeout: number
        }
    }
}

declare namespace AccountService {
    export type AccountId = GUID
    interface AccountDefinition<TProp = any, TRole = string> {
        id: string
        name: string
        displayName: string
        role?: TRole
        props?: TProp
    }
    type Account = AccountDefinition<any, string>
}

declare namespace AuthenticationService {
    export type Settings = {
        authentication?: {
            ttl?: number
        }
    }
    export interface AuthenticationData<TType extends string = string, TMeta = any> {
        // We use uuid to identify unique authentication
        // suggest using hash of the unique criteria from TMeta
        guid?: string
        accountId?: string
        createdAt?: number
        active?: boolean
        type: TType
        meta: TMeta
    }
    export type CommonAuthenticationData = AuthenticationData<"common", {
        name: string
        displayName: string
        email: string
        password?: string
        passwordHash?: string
        active: boolean
    }>
}

declare namespace KeyValueStorage {
    export type Settings = {
        kvs: {
            collection: string
        }
    }
}

declare namespace BlobService {
    export type Settings = {
        blob: {
            route?: string
            storage: string
        }
    }
}

declare namespace ExpressService {
    export type Settings = {
        http?: {
            port?: number
            host?: string
            offline?: boolean
            limit: {
                jsonInKb?: number
                bodyInKb?: number
            }
            staticResources?: {
                path: string
                route: string
            }[]
        }
    }
}

declare namespace RouteSessionService {
    export type Settings = {
        http?: {
            session?: {
                secret?: string
                maxAge?: number
                resave?: boolean
                saveUninitialized?: boolean
            }
        }
    }
}
declare namespace WechatService {
    export type Settings = {
        wechat: {
            appId: string
            secret: string
            domain?: {
                filename: string
                content: string
            }
            // used for redirect back from wechat
            // should contain the domain and protocol
            baseUrl: string
        }
    }
}
declare namespace WechatPaymentService {
    export type Settings = {
        wechat: {
            payment: {
                merchantId: string
                key: string
                notificationUrl: string
            }
        }
    }
}

declare namespace EndlessService {
    export type Settings = {
        endless?: {
            count?: number
        }
    }
    export type EndlessQueryMeta = {
        offset?: number,
        count?: number,
    }
    export type EndlessQuery<T = any> = {
        offset: number,
        count: number,
        criteria?: {
            [K in keyof T]?: any
        }
        sortField?: string,
        sortOrder?: number
    }
}

declare namespace MailService {
    type Settings = {
        mail: {
            provider: "sendgrid"
            sendgrid?: {
                apiKey: string
            }
        }
    }
    type EmailAddress = string
    type HTML = string
    type Mail = {
        title: string
        name: string
        content: HTML
        fallbackText?: string
        fromName?: string
        toName?: string
    }
}

declare namespace PersistentService {
    type Settings = {
        persistent: {
            storage: string
            storageName: string
        }
    }
    interface Persistentable {
        name: string
        toJSON(): any
        fromJSON(data: any)
    }
}
